@@ -17,23 +17,41 @@ except: # pragma: no cover
 
 # Set the defaults in a re-usable way
 default_args = argparse.Namespace(
-        targets      = ['.'], # Not in configs
-        subprocesses = 1,
-        html         = False,
-        termcolor    = None,
-        notermcolor  = None,
-        debug        = 0,
-        help         = False, # Not in configs
-        logging      = False,
-        version      = False,
-        verbose      = 1,
-        config       = None,  # Not in configs
-        run_coverage = False,
-        omit         = None,
+        targets         = ['.'], # Not in configs
+        subprocesses    = 1,
+        html            = False,
+        termcolor       = None,
+        notermcolor     = None,
+        debug           = 0,
+        help            = False, # Not in configs
+        logging         = False,
+        version         = False,
+        verbose         = 1,
+        config          = None,  # Not in configs
+        run_coverage    = False,
+        omit            = None,
+        completion_file = False,
+        completions     = False,
+        options         = False,
         )
 
 
+
+class StoreOpt():
+
+
+    def __init__(self):
+        self.options = []
+        self.options = []
+
+
+    def __call__(self, action):
+        self.options.extend(action.option_strings[0:2])
+
+
+
 def main(testing=False, coverage_testing=False):
+    store_opt = StoreOpt()
     parser = argparse.ArgumentParser(
             add_help=False,
             description=
@@ -42,6 +60,16 @@ Green is a clean, colorful test runner for Python unit tests.
 """.rstrip(),
             epilog=
 """
+ENABLING SHELL COMPLETION
+
+  To enable bash- or zsh-completion, add the line below to the end of your
+  .bashrc or .zshrc file (or equivalent config file):
+
+    which green > /dev/null && source "$( green --completion-file )"
+
+  Warning!  Generating a completion list actually discovers and loads tests
+  -- this can be very slow if you run it in huge directories!
+
 CONFIG FILES
 
   Green will look for and process three config files if found:
@@ -52,8 +80,8 @@ CONFIG FILES
   Config file format is simply "option = value" on separate lines.  "option" is
   the same as the long options above, just without the "--".
 
-  Most values should be "True" or "False".  Accumulated values (verbose, debug) should
-  be specified as integers ("-vv" would be "verbose = 2").
+  Most values should be "True" or "False".  Accumulated values (verbose, debug)
+  should be specified as integers ("-vv" would be "verbose = 2").
 
   Example:
 
@@ -74,7 +102,8 @@ CONFIG FILES
         only tests accessible from introspection of the object will be
         loaded."""))
     concurrency_args = parser.add_argument_group("Concurrency Options")
-    concurrency_args.add_argument('-s', '--subprocesses', action='store',
+    store_opt(
+        concurrency_args.add_argument('-s', '--subprocesses', action='store',
             type=int, metavar='NUM',
             help="Number of subprocesses to use to run tests.  Note that your "
             "tests need to be written to avoid using the same resources (temp "
@@ -82,43 +111,57 @@ CONFIG FILES
             "well. Default is 1, meaning try to autodetect the number of CPUs "
             "in the system.  1 will disable using subprocesses.  Note that for "
             "trivial tests (tests that take < 1ms), running everything in a "
-            "single process may be faster.")
+            "single process may be faster."))
     format_args = parser.add_argument_group("Format Options")
-    format_args.add_argument('-m', '--html', action='store_true',
-        help="HTML5 format.  Overrides terminal color options if specified.")
-    format_args.add_argument('-t', '--termcolor', action='store_true',
-        help="Force terminal colors on.  Default is to autodetect.")
-    format_args.add_argument('-T', '--notermcolor', action='store_true',
-        help="Force terminal colors off.  Default is to autodetect.")
+    store_opt(format_args.add_argument('-m', '--html', action='store_true',
+        help="HTML5 format.  Overrides terminal color options if specified."))
+    store_opt(format_args.add_argument('-t', '--termcolor', action='store_true',
+        help="Force terminal colors on.  Default is to autodetect."))
+    store_opt(
+        format_args.add_argument('-T', '--notermcolor', action='store_true',
+        help="Force terminal colors off.  Default is to autodetect."))
     out_args = parser.add_argument_group("Output Options")
-    out_args.add_argument('-d', '--debug', action='count',
+    store_opt(out_args.add_argument('-d', '--debug', action='count',
         help=("Enable internal debugging statements.  Implies --logging.  Can "
-        "be specified up to three times for more debug output."))
-    out_args.add_argument('-h', '--help', action='store_true',
-        help="Show this help message and exit.")
-    out_args.add_argument('-l', '--logging', action='store_true',
+        "be specified up to three times for more debug output.")))
+    store_opt(out_args.add_argument('-h', '--help', action='store_true',
+        help="Show this help message and exit."))
+    store_opt(out_args.add_argument('-l', '--logging', action='store_true',
         help="Don't configure the root logger to redirect to /dev/null, "
-        "enabling internal debugging output")
-    out_args.add_argument('-V', '--version', action='store_true',
-        help="Print the version of Green and Python and exit.")
-    out_args.add_argument('-v', '--verbose', action='count',
+        "enabling internal debugging output"))
+    store_opt(out_args.add_argument('-V', '--version', action='store_true',
+        help="Print the version of Green and Python and exit."))
+    store_opt(out_args.add_argument('-v', '--verbose', action='count',
         help=("Verbose. Can be specified up to three times for more verbosity. "
-        "Recommended levels are -v and -vv."))
+        "Recommended levels are -v and -vv.")))
     other_args = parser.add_argument_group("Other Options")
-    other_args.add_argument('-c', '--config', action='store',
+    store_opt(other_args.add_argument('-c', '--config', action='store',
         metavar='FILE', help="Use this config file instead of the one pointed "
-        "to by environment variable GREEN_CONFIG or the default ~/.green")
+        "to by environment variable GREEN_CONFIG or the default ~/.green"))
     cov_args = parser.add_argument_group(
         "Coverage Options ({})".format(coverage_version))
-    cov_args.add_argument('-r', '--run-coverage', action='store_true',
-        help=("Produce coverage output."))
-    cov_args.add_argument('-o', '--omit', action='store',
+    store_opt(cov_args.add_argument('-r', '--run-coverage', action='store_true',
+        help=("Produce coverage output.")))
+    store_opt(cov_args.add_argument('-o', '--omit', action='store',
         metavar='PATTERN',
         help=("Comma-separated file-patterns to omit from coverage.  Default "
             "is something like '*/test*,*/termstyle*,*/mock*,*(temp "
-            "dir)*,*(python system packages)*'"))
-    parser.set_defaults(**(dict(default_args._get_kwargs())))
+            "dir)*,*(python system packages)*'")))
+    # These options are used by bash-completion and zsh completion.
+    integration_args = parser.add_argument_group("Integration Options")
+    store_opt(integration_args.add_argument('--completion-file',
+        action='store_true', help=("Location of the bash- and zsh-completion "
+            "file.  To enable bash- or zsh-completion, see ENABLING SHELL "
+            "COMPLETION below."
+            )))
+    store_opt(integration_args.add_argument('--completions',
+        action='store_true',
+        help=("Output possible completions of the given target.  Used by bash- "
+        "and zsh-completion.")))
+    store_opt(integration_args.add_argument('--options', action='store_true',
+        help="Output all options.  Used by bash- and zsh-completion."))
 
+    parser.set_defaults(**(dict(default_args._get_kwargs())))
     args = parser.parse_args()
 
     # Unfortunately we can't fully cover the config module (the global part of
@@ -194,6 +237,22 @@ CONFIG FILES
             termcolor=args.termcolor, subprocesses=args.subprocesses,
             run_coverage=args.run_coverage, omit=omit)
 
+    # Location of shell completion file
+    if args.completion_file:
+        print(os.path.join(os.path.dirname(__file__), 'shell_completion.sh'))
+        return 0
+
+    # Argument-completion for bash and zsh (for test-target completion)
+    if args.completions:
+        from green.loader import getCompletions
+        print(getCompletions(args.targets))
+        return 0
+
+    # Option-completion for bash and zsh
+    if args.options:
+        print('\n'.join(sorted(store_opt.options)))
+        return 0
+
     # Add debug logging for stuff that happened before this point here
     if config.files_loaded:
         debug("Loaded config file(s): {}".format(
@@ -201,22 +260,22 @@ CONFIG FILES
 
     # Discover/Load the TestSuite
     if testing:
-        tests = None
+        test_suite = None
     else:
-        tests = loadTargets(args.targets)
+        test_suite = loadTargets(args.targets)
 
     # We didn't even load 0 tests...
-    if not tests:
+    if not test_suite:
         debug(
             "No test loading attempts succeeded.  Created an empty test suite.")
-        tests = unittest.suite.TestSuite()
+        test_suite = unittest.suite.TestSuite()
 
-    # Actually run the tests
+    # Actually run the test_suite
     if testing:
         result = lambda: None
         result.wasSuccessful = lambda: 0
     else:
-        result = runner.run(tests) # pragma: no cover
+        result = runner.run(test_suite) # pragma: no cover
 
     if args.run_coverage and ((not testing) or coverage_testing):
         stream.writeln()
